;***************************************************************;
; chimes.asm         (mpasm)                                    ;
; Plays Westminster Chimes for each quarter hour.               ;
; Separate switch for each quarter hour.                        ;
; Written 11 Aug 1999 by Richard M. Bender, W3SYY               ;
; Ports:                                                        ;
; B4=key-Qtr4, B3=key-Qtr3, B2=key-Qtr2, B1=key-Qtr1            ;
; B0 - speaker    (key to ground to activate)                   ;
; 4 MHz resonator clock                                         ;
;***************************************************************;

        list      p=16F84, r=dec
        __config  3FF1H      ; define XT (4MHz resonator) clock
                             ; no Watchdog or Power-up Timers

portb   equ     6            ; define port B
Q4      equ     4            ; PB4 key for 4th quarter
Q3      equ     3            ; PB3 key for 3rd quarter
Q2      equ     2            ; PB2 key for 2nd quarter
Q1      equ     1            ; PB1 key for 1st quarter
Spkr    equ     0            ; PB0 for Speaker
count   equ     12           ; pulse length count/2
count0  equ     13           ; constant 2 for above
count1  equ     14           ; delay counts

        org     0

start   movlw   0            ; clear acc
        option               ; set options to 0
        clrf    portb        ; clear port bits
        movlw   11111110B    ; bit0=output, others=input
        tris    portb        ; initialize port B

loop    btfss   portb,Q4     ; check for key down - 4th quarter
        goto    qtr_4        ; do 4th quarter
        btfss   portb,Q3     ; check for key down - 3rd quarter
        goto    qtr_3        ; do 3rd quarter
        btfss   portb,Q2     ; check for key down - 2nd quarter
        goto    qtr_2        ; do 2nd quarter
        btfsc   portb,Q1     ; check for key down - 1st quarter
        goto    loop         ; no keys down, loop again


; first quarter hour selection - one measure (4 notes)
qtr_1   call    measure_a    ; one measure
        goto    loop

; second quarter hour selection - two measures (8 notes)
qtr_2   call    measure_b    ; two measures
        goto    loop

; third quarter hour selection - three measures (12 notes)
qtr_3   call    measure_c       ; two measures
        call    measure_a       ; one measure
        goto    loop

; fourth quarter hour selection - four measures (16 notes)
qtr_4   call    measure_b       ; first two measures
        call    measure_c       ; last two measures
        goto    loop

measure_a
        call    note_E          
        call    note_D
        call    note_C
        call    note_G
        call    note_G
        call    half_sec
        return

measure_b                    
        call    note_C       ; "Lord through this hour,"
        call    note_E
        call    note_D
        call    note_G
        call    note_G
        call    half_sec     
        call    note_C       ; "Be Thou our guide"
        call    note_D
        call    note_E
        call    note_C
        call    note_C
        call    half_sec
        return

measure_c
        call    note_E       ; "So, by Thy power"
        call    note_C
        call    note_D
        call    note_G
        call    note_G
        call    half_sec
        call    note_G       ; "No foot shall slide."
        call    note_D
        call    note_E
        call    note_C
        call    note_C
        call    half_sec
        return

half_sec        ; half second silent delay - 500,000 usec
        movlw   250
        movwf   count        ; 250 * 2000 us = 500,000 us = 1/2 sec
hs_l    movlw   200
        movwf   count1
hs_l1   nop                  ; 10us loop x 200 = 2000 us 
        nop
        nop
        nop
        nop
        nop
        nop
        decfsz  count1,f
        goto    hs_l1
        decfsz  count,f
        goto    hs_l
        return

note_G
        movlw   196          ; 196 pulses per half second (392 Hz)
        movwf   count
ng_l    movlw   1276/5       ; count for half cycle time
        movwf   count1
        bsf     portb,Spkr   ; set speaker bit high
        call    delay
        movlw   1276/5       ; second half cycle
        movwf   count1
        call    delay
        decfsz  count,f
        goto    ng_l         ; do another cycle
        return

note_C 
        movlw   2            ; needed to get total pulse count
        movwf   count0
notec   movlw   262/2        ; 262 pulses per half second (523.25 Hz)
        movwf   count
ng_c    movlw   956/5        ; count for half cycle time
        movwf   count1
        bsf     portb,Spkr   ; set speaker bit high
        call    delay
        movlw   956/5        ; second half cycle
        movwf   count1
        call    delay
        decfsz  count,f
        goto    ng_c         ; do another cycle
        decfsz  count0,f
        goto    notec        ; do second half of total pulse count
        return

note_D
        movlw   2            ; needed to get total pulse count
        movwf   count0
noted   movlw   294/2        ; 294 pulses per half second (587.83 Hz)
        movwf   count
ng_d    movlw   851/5        ; count for half cycle time
        movwf   count1
        bsf     portb,Spkr   ; set speaker bit high
        call    delay
        movlw   851/5        ; second half cycle
        movwf   count1
        call    delay
        decfsz  count,f
        goto    ng_d         ; do another cycle
        decfsz  count0,f
        goto    noted        ; do second half of total pulse count
        return

note_E
        movlw   2            ; needed to get total pulse count
        movwf   count0
notee   movlw   330/2        ; 330 pulses per half second (659.26 Hz)
        movwf   count
ng_e    movlw   758/5        ; count for half cycle time
        movwf   count1
        bsf     portb,Spkr   ; set speaker bit high
        call    delay
        movlw   758/5        ; second half cycle
        movwf   count1
        call    delay
        decfsz  count,f
        goto    ng_e         ; do another cycle
        decfsz  count0,f
        goto    notee        ; do second half of total pulse count
        return

delay                        ; delay is count1 * 5 us
        nop                  ; 5us loop x count1 = delay
        nop
        decfsz  count1,f
        goto    delay
        bcf     portb,Spkr      ; set speaker bit low
        return

        end

